@@ -5,12 +5,14 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
 the GNU General Public License as published by the Free Software Foundation.
 Either version 2 of the License, or (at your option) any later version.
 */
+
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,6 +32,11 @@ void set_alarmhandler(char* handler,int level)
   _alarmlevel=level;
 }
 
+void alarm_handler0(int severity, char *devicename, char *text)
+{
+  alarm_handler(severity, devicename, "%s", text);
+}
+
 void alarm_handler(int severity,char* devicename,char* format, ...)
 {
   va_list argp;
@@ -37,6 +44,7 @@ void alarm_handler(int severity,char* devicename,char* format, ...)
   char cmdline[PATH_MAX+1024];
   char timestamp[40];
   time_t now;
+
   if (_alarmhandler[0])
   {
     va_start(argp,format);
@@ -47,7 +55,7 @@ void alarm_handler(int severity,char* devicename,char* format, ...)
       time(&now);
       strftime(timestamp,sizeof(timestamp),"%Y-%m-%d %H:%M:%S",localtime(&now));
       snprintf(cmdline,sizeof(cmdline),"%s ALARM %s %i %s \"%s\"",_alarmhandler,timestamp,severity,devicename,text);      
-      my_system(cmdline);
+      my_system(cmdline, "alarmhandler");
     }  
   }
 }
