@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -27,29 +28,14 @@ void cutcomment(char*  text)
   if (cp!=0)
     *cp=0;
   laenge=strlen(text);
-  while ((laenge>0)&&(text[laenge-1]<=' '))
+  // 3.1beta7: this was dropping scandinavic characters, unsigned test added:
+  while (laenge > 0 && ((unsigned char)text[laenge -1] <= (unsigned char)' '))
   {
     text[laenge-1]=0;
     laenge--;
   }
 }
 
-int yesno( char*  value)
-{
-  if ((value[0]=='1') ||
-      (value[0]=='y') ||
-      (value[0]=='Y') ||
-      (value[0]=='t') ||
-      (value[0]=='T') ||
-      ((value[1]=='n') && (
-        (value[0]=='o') ||
-        (value[0]=='O'))
-      ))
-    return 1;
-  else
-    return 0;
-}
-
 int getsubparam_delim(char*  parameter, 
                       int n, 
                       char*  subparam,  int size_subparam,
@@ -118,21 +104,29 @@ int splitline( char*  source,
 int gotosection(FILE* file,  char*  name)
 {
   char line[PATH_MAX+32];
-  int Length;
+  //int Length;
   char* posi;
 
   fseek(file,0,SEEK_SET);
   while (fgets(line,sizeof(line),file))
   {
     cutcomment(line);
-    Length=strlen(line);
-    if (Length>2)
+
+    // 3.1beta7: clean whitespaces:
+    while (is_blank(*line))
+      strcpy(line, line +1);
+
+    //Length=strlen(line);
+    //if (Length>2)
+    if (*line)
     {
       posi=strchr(line,']');
       if ((line[0]=='[') && posi)
+      {// 3.1beta7: added brackets, should be a block, otherwise name is still tested.
         *posi=0;
         if (strcmp(line+1,name)==0)
 	  return 1;
+      }
     }
   }
   return 0;
@@ -143,13 +137,20 @@ int my_getline(FILE* file,
 	    char*  value,  int size_value)
 {
   char line[PATH_MAX+32];
-  int Length;
+  //int Length;
 
   while (fgets(line,sizeof(line),file))
   {
     cutcomment(line);
-    Length=strlen(line);
-    if (Length>2)
+    //Length=strlen(line);
+
+    // 3.1beta7: clean whitespaces:
+    while (is_blank(*line))
+      strcpy(line, line +1);
+
+    // 3.1beta7: lines with one or two illegal characters were not reported:
+    //if (Length>2)
+    if (*line)
     {
       if (line[0]=='[')
         return 0;
