@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -25,6 +26,11 @@ Either version 2 of the License, or (at your option) any later version.
 // Change this if other than Euro character is wanted, like '?' or '$'.
 #define GSM_CURRENCY_SYMBOL_TO_ISO 0xA4
 
+// For incoming character 0x09 conversion:
+// (some reference: ftp://www.unicode.org/Public/MAPPINGS/ETSI/GSM0338.TXT)
+// Uncomment this if you want that C-CEDILLA is represented as small c-cedilla:
+//#define INCOMING_SMALL_C_CEDILLA
+
 // iso = ISO8859-15 (you might change the table to any other 8-bit character set)
 // sms = sms character set used by mobile phones
 
@@ -38,7 +44,13 @@ char charset[] = { '@' , 0x00, // COMMERCIAL AT
 		   0xF9, 0x06, // LATIN SMALL LETTER U WITH GRAVE
 		   0xEC, 0x07, // LATIN SMALL LETTER I WITH GRAVE
 		   0xF2, 0x08, // LATIN SMALL LETTER O WITH GRAVE
+
+#ifdef INCOMING_SMALL_C_CEDILLA
+		   0xE7, 0x09, // LATIN SMALL LETTER C WITH CEDILLA
+#else
 		   0xC7, 0x09, // LATIN CAPITAL LETTER C WITH CEDILLA
+#endif
+
 		   0x0A, 0x0A, // LF
 		   0xD8, 0x0B, // LATIN CAPITAL LETTER O WITH STROKE
 		   0xF8, 0x0C, // LATIN SMALL LETTER O WITH STROKE
@@ -203,6 +215,8 @@ char ext_charset[] = { 0x0C, 0x0A, // <FF>
 	             };
 
 
+// This table is used for outgoing (to GSM) conversion only:
+
 char iso_8859_15_chars[] =
 {
 	0x60, 0x27, // GRAVE ACCENT --> APOSTROPHE
@@ -236,6 +250,7 @@ char iso_8859_15_chars[] =
 	0xC1, 0x41, // LATIN CAPITAL LETTER A WITH ACUTE --> A
 	0xC2, 0x41, // LATIN CAPITAL LETTER A WITH CIRCUMFLEX --> A
 	0xC3, 0x41, // LATIN CAPITAL LETTER A WITH TILDE --> A
+	0xC7, 0x09, // LATIN CAPITAL LETTER C WITH CEDILLA --> 0x09 (LATIN CAPITAL LETTER C WITH CEDILLA)
 	0xC8, 0x45, // LATIN CAPITAL LETTER E WITH GRAVE --> E
 	0xCA, 0x45, // LATIN CAPITAL LETTER E WITH CIRCUMFLEX --> E
 	0xCB, 0x45, // LATIN CAPITAL LETTER E WITH DIAERESIS --> E
@@ -648,6 +663,7 @@ int iso2utf8_file(FILE *fp, char *ascii, int userdatalength)
   return result;
 }
 
+/*
 int iso2gsm(char* source, int size, char* destination, int max)
 {
   int table_row;
@@ -707,6 +723,7 @@ int iso2gsm(char* source, int size, char* destination, int max)
   destination[dest_count]=0;
   return dest_count;
 }
+*/
 
 int gsm2iso(char* source, int size, char* destination, int max)
 {
@@ -762,6 +779,7 @@ int gsm2iso(char* source, int size, char* destination, int max)
   return dest_count;
 }
 
+/*
 int unicode2sms(char* source, int size, char* destination, int max)
 {
   int foundstart=0;
@@ -798,6 +816,7 @@ int unicode2sms(char* source, int size, char* destination, int max)
   destination[dest_count]=0;
   return dest_count;
 }
+*/
 
 int decode_ucs2(char *buffer, int len)
 {
