@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -15,6 +16,12 @@ Either version 2 of the License, or (at your option) any later version.
 #ifndef CHARSET_H
 #define CHARSET_H
 
+char logch_buffer[8192];
+
+// Logging is not used externally, but it's placed to the end of source file.
+void logch(char* format, ...);
+char prch(char ch);
+
 // Both functions return the size of the converted string
 // max limits the number of characters to be written into
 // destination
@@ -24,9 +31,12 @@ Either version 2 of the License, or (at your option) any later version.
 
 int gsm2iso(char* source, int size, char* destination, int max);
 
-int iso2gsm(char* source, int size, char* destination, int max);
+int iso_utf8_2gsm(char* source, int size, char* destination, int max);
+int iso2utf8_file(FILE *fp, char *ascii, int userdatalength);
 
-int unicode2sms(char* source, int size, char* destination, int max);
+//int iso2gsm(char* source, int size, char* destination, int max);
+
+//int unicode2sms(char* source, int size, char* destination, int max);
 int decode_ucs2(char *buffer, int len);
 
 #endif
