@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -31,6 +32,82 @@ Either version 2 of the License, or (at your option) any later version.
 #include "logging.h"
 #include "alarm.h"
 
+int yesno(char *value)
+{
+  extern char yes_chars[];
+  extern char no_chars[];
+  char *p, *p2;
+
+  if (*yes_chars)
+  {
+    p = yes_chars;
+    while (*p)
+    {
+      if (!(p2 = strchr(p, '\'')))
+        break;
+      if (!strncmp(value, p, (int)(p2 -p)))
+        return 1;
+      p = p2 +1;
+    }
+  }
+
+  if (*no_chars)
+  {
+    p = no_chars;
+    while (*p)
+    {
+      if (!(p2 = strchr(p, '\'')))
+        break;
+      if (!strncmp(value, p, (int)(p2 -p)))
+        return 0;
+      p = p2 +1;
+    }
+  }
+
+  if ((value[0]=='1') ||
+      (value[0]=='y') ||
+      (value[0]=='Y') ||
+      (value[0]=='t') ||
+      (value[0]=='T') ||
+      ((value[1]=='n') && (
+        (value[0]=='o') ||
+        (value[0]=='O'))
+      ))
+    return 1;
+  else
+    return 0;
+}
+
+int yesno_check(char*  value)
+{
+  // This version is used to check config file values.
+  int result = -1;
+
+  if ((value[0]=='1') ||
+      (value[0]=='y') ||
+      (value[0]=='Y') ||
+      (value[0]=='t') ||
+      (value[0]=='T') ||
+      ((value[1]=='n') && (
+        (value[0]=='o') ||
+        (value[0]=='O'))
+      ))
+    result = 1;
+  else
+  if ((value[0]=='0') ||
+      (value[0]=='n') ||
+      (value[0]=='N') ||
+      (value[0]=='f') ||
+      (value[0]=='F') ||
+      ((value[1]=='f') && (
+        (value[0]=='o') ||
+        (value[0]=='O'))
+      ))
+    result = 0;
+
+  return result;
+}
+
 char *cut_ctrl(char* message) /* removes all ctrl chars */
 {
   // 3.0.9: use dynamic buffer to avoid overflow:
@@ -45,7 +122,8 @@ char *cut_ctrl(char* message) /* removes all ctrl chars */
   {
     for (possource=0; possource<=count; possource++)
     {
-      if ((message[possource]>=' ') || (message[possource]==0))
+      // 3.1beta7: added unsigned test:
+      if (((unsigned char)message[possource] >= (unsigned char)' ') || (message[possource]==0))
         tmp[posdest++]=message[possource];
     }
     strcpy(message,tmp);
@@ -195,6 +273,8 @@ int movefilewithdestlock_new(char* filename, char* directory, int keep_fname, in
     FILE *fpnew;
     size_t n;
     char *p;
+    extern const char *HDR_OriginalFilename;
+    extern char HDR_OriginalFilename2[];
 
     strcpy(line, prefix);
     if (*line)
@@ -207,11 +287,11 @@ int movefilewithdestlock_new(char* filename, char* directory, int keep_fname, in
     unlink(newname);
     if (!result)
     {
-      if ((fpnew = fopen(newname, "w")) == NULL)
+      if (!(fpnew = fopen(newname, "w")))
         result = 2;
       else
       {
-        if ((fp = fopen(filename, "r")) == NULL)
+        if (!(fp = fopen(filename, "r")))
         {
           fclose(fpnew);
           unlink(newname);
@@ -223,10 +303,11 @@ int movefilewithdestlock_new(char* filename, char* directory, int keep_fname, in
           {
             if (line_is_blank(line))
             {
-              if (store_original_fname)
+              if (store_original_fname && *HDR_OriginalFilename2 != '-')
               {
                 p = strrchr(filename, '/');
-                fprintf(fpnew, "Original_filename: %s\n", (p)? p +1 : filename);
+                fprintf(fpnew, "%s %s\n", (*HDR_OriginalFilename2)? HDR_OriginalFilename2 : HDR_OriginalFilename,
+                        (p)? p +1 : filename);
               }
               in_headers = 0;
             }
@@ -259,7 +340,7 @@ int movefilewithdestlock_new(char* filename, char* directory, int keep_fname, in
   }
 }
 
-void cutspaces(char*  text)
+char *cutspaces(char *text)
 {
   int count;
   int Length;
@@ -284,6 +365,8 @@ void cutspaces(char*  text)
     text[Length-1]=0;
     Length--;
   }
+
+  return text;
 }
 
 void cut_emptylines(char*  text)
@@ -318,23 +401,43 @@ int is_highpriority(char *filename)
   FILE *fp;
   char line[256];
   int result = 0;
+  // 3.1beta7: language settings used:
+  extern const char *HDR_Priority;
+  extern char HDR_Priority2[];
+  int hlen;
+  int hlen2;
+  char *compare;
+  char *compare2 = 0;
 
-  if ((fp = fopen(filename, "r")) != NULL)
+  // get_header() and test_header() can be moved to this file,
+  // but this is faster:
+  if (*HDR_Priority2 && strcmp(HDR_Priority2, "-"))
   {
-    while (fgets(line, sizeof(line), fp) != NULL)
+    if (*HDR_Priority2 == '-')
+      compare2 = HDR_Priority2 +1;
+    else
+      compare2 = HDR_Priority2;
+    hlen2 = strlen(compare2);
+  }
+
+  compare = (char *)HDR_Priority;
+  hlen = strlen(compare);
+
+  if ((fp = fopen(filename, "r")))
+  {
+    while (!result && fgets(line, sizeof(line), fp))
     {
       if (line_is_blank(line))
         break;
 
-      if (strstr(line,"Priority:")==line)
+      if ((compare2 && strncmp(line, compare2, hlen2) == 0) ||
+          strncmp(line, compare, hlen) == 0)
       {
-        memmove(line,line+9,strlen(line)-8);
-        cutspaces(line);
-        if (strcasecmp(line,"HIGH")==0)
-        {
+        cutspaces(strcpy(line, line +hlen));
+        if (!strcasecmp(line,"HIGH"))
+          result = 1;
+        else if (yesno(line) == 1)
           result = 1;
-          break;
-        }
       }
     }
     fclose(fp);
@@ -347,7 +450,7 @@ int file_is_writable(char *filename)
   int result = 0;
   FILE *fp;
 
-  if ((fp = fopen(filename, "a")) != NULL)
+  if ((fp = fopen(filename, "a")))
   {
     result = 1;
     fclose(fp);
@@ -373,7 +476,7 @@ int getpdufile(char *filename)
           if (file_is_writable(filename))
             result = 1;
     }
-    else if (strchr(filename, '.') == NULL)
+    else if (!strchr(filename, '.'))
     {
       if (stat(filename, &statbuf) == 0)
       {
@@ -456,6 +559,46 @@ int getfile(char *dir, char *filename)
               if (!check_access(tmpname))
                 chmod(tmpname, 0766);
 
+            // 3.1:
+            if (!os_cygwin && !file_is_writable(tmpname))
+            {
+              int result = 1;
+              char tmp_filename[PATH_MAX +7];
+              FILE *fp;
+              FILE *fptmp;
+              char line[1024];
+              size_t n;
+
+              sprintf(tmp_filename, "%s.XXXXXX", tmpname);
+              close(mkstemp(tmp_filename));
+              unlink(tmp_filename);
+
+              if ((fptmp = fopen(tmp_filename, "w")) == NULL)
+                result = 0;
+              else
+              {
+                if ((fp = fopen(tmpname, "r")) == NULL)
+                  result = 0;
+                else
+                {
+                  while ((n = fread(line, 1, sizeof(line), fp)) > 0)
+                    fwrite(line, 1, n, fptmp);
+
+                  fclose(fp);
+                }
+
+                fclose(fptmp);
+
+                if (result)
+                {
+                  unlink(tmpname);
+                  rename(tmp_filename, tmpname);
+                }
+                else
+                  unlink(tmp_filename);
+              }
+            }
+
             if (!file_is_writable(tmpname))
             {
               int report = 1;
@@ -886,7 +1029,7 @@ int is_executable(char *filename)
       {
         if ((n = getgroups(0, NULL)) > 0)
         {
-          if ((g = (gid_t *)malloc(n * sizeof(gid_t))) != NULL)
+          if ((g = (gid_t *)malloc(n * sizeof(gid_t))))
           {
             if ((n = getgroups(n, g)) > 0)
             {
@@ -936,7 +1079,7 @@ int check_access(char *filename)
       {
         if ((n = getgroups(0, NULL)) > 0)
         {
-          if ((g = (gid_t *)malloc(n * sizeof(gid_t))) != NULL)
+          if ((g = (gid_t *)malloc(n * sizeof(gid_t))))
           {
             if ((n = getgroups(n, g)) > 0)
             {
