@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -23,8 +24,8 @@ Either version 2 of the License, or (at your option) any later version.
 #include <fcntl.h>
 #include <unistd.h>
 
-int  Filehandle;
-int  Level;
+int Filehandle = -1;
+int Level;
 
 int openlogfile(char* filename,char* progname,int facility,int level)
 {
@@ -64,7 +65,10 @@ int openlogfile(char* filename,char* progname,int facility,int level)
 void closelogfile()
 {
   if (Filehandle>=0)
+  {
     close(Filehandle);
+    Filehandle = -1;
+  }
 }
 
 void writelogfile0(int severity, char *progname, char *text)
