@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -177,7 +178,8 @@ _gsm_general_error gsm_cms_errors[] =
   {331,	"No network service"},
   {332,	"Network timeout"},
   {340,	"No +CNMA acknowledgement expected"},
-  {500,	"Unknown error"}
+  {500,	"Unknown error"},
+  {515, "Please wait, service is not available, init in progress"}
 };
 
 char *get_gsm_cme_error(int code)
@@ -210,18 +212,16 @@ char *get_gsm_error(char *answer)
 
   if (answer && *answer)
   {
-    if ((p = strstr(answer, "+CME ERROR: ")) != NULL)
+    if ((p = strstr(answer, "+CME ERROR: ")))
       return get_gsm_cme_error(atoi(p +12));
-    if ((p = strstr(answer, "+CMS ERROR: ")) != NULL)
+    if ((p = strstr(answer, "+CMS ERROR: ")))
       return get_gsm_cms_error(atoi(p +12));
   }
 
   return "";
 }
 
-// The following functions are for internal use of put_command and therefore not in the modeminit.h.
-
-int write_to_modem(int modem,char* modemname,int send_delay,char* command,int timeout, int log_command, int print_error)
+int write_to_modem(int modem, char *modemname, int send_delay, char *command, int timeout, int log_command, int print_error)
 {
   int status=0;
   int timeoutcounter=0;
@@ -280,7 +280,7 @@ int write_to_modem(int modem,char* modemname,int send_delay,char* command,int ti
 // least 1 character and then the modem is quiet for timeout*0.1s.
 // The answer might contain already a string. In this case, the answer 
 // gets appended to this string.
-int read_from_modem(int modem, char* modemname, int send_delay, char* answer,int max,int timeout)
+int read_from_modem(int modem, char *modemname, int send_delay, char *answer, int max, int timeout)
 {
   int count=0;
   int got=0;
@@ -321,22 +321,25 @@ int read_from_modem(int modem, char* modemname, int send_delay, char* answer,int
   return success;
 }
 
-
-// Exported functions:
-
-
-int put_command(int modem, char* modemname, int send_delay,char* command,char* answer,int max,int timeout,char* expect)
+// 3.1beta7: Not waiting any answer if answer is NULL. Return value is then 1/0.
+int put_command(int modem, char *modemname, int send_delay, char *command, char *answer, int max, int timeout, char *expect)
 {
   char loganswer[2048];
-  int timeoutcounter=0;
+  int timeoutcounter = 0;
   regex_t re;
-  int got_timeout=1;
+  int got_timeout = 1;
+  int regex_allocated = 0;
 
   // compile regular expressions
-  if (expect && expect[0] && (regcomp(&re,expect, REG_EXTENDED|REG_NOSUB) != 0)) 
-  {	  
-    fprintf(stderr,"Programming error: Modem answer %s is not a valid regepr\n",expect);
-    exit(1);
+  if (expect && expect[0])
+  {
+    if (regcomp(&re, expect, REG_EXTENDED|REG_NOSUB) != 0)
+    {	  
+      fprintf(stderr, "Programming error: Expected answer %s is not a valid regepr\n", expect);
+      writelogfile(LOG_CRIT, modemname, "Programming error: Expected answer %s is not a valid regepr", expect);
+      exit(1);
+    }
+    regex_allocated = 1;
   }  
 
   // clean input buffer
@@ -346,52 +349,84 @@ int put_command(int modem, char* modemname, int send_delay,char* command,char* a
   tcflush(modem,TCIFLUSH);
   
   // send command
-  if (write_to_modem(modem,modemname,send_delay,command,30, 1, 0)==0)
+  if (write_to_modem(modem, modemname, send_delay, command, 30, 1, 0) == 0)
   {
     t_sleep(errorsleeptime);
     // Free memory used by regexp
-    regfree(&re);
+    if (regex_allocated)
+      regfree(&re);
     return 0;
   }
-  writelogfile(LOG_DEBUG,modemname,"Command is sent, waiting for the answer");
-
-  // wait for the modem-answer 
-  answer[0]=0;
-  timeoutcounter=0;
-  do
+ 
+  if (!answer)
+    writelogfile(LOG_DEBUG, modemname, "Command is sent");
+  else
   {
-    read_from_modem(modem,modemname,send_delay,answer,max,2);  // One read attempt is 200ms
+    writelogfile(LOG_DEBUG, modemname, "Command is sent, waiting for the answer");
 
-    // check if it's the expected answer
-    if (expect && expect[0] && (regexec(&re, answer, (size_t) 0, NULL, 0)==0))
+    // wait for the modem-answer 
+    answer[0] = 0;
+    timeoutcounter = 0;
+    do
     {
-      got_timeout = 0;
-      put_command_timeouts = 0;
-      break;
+      read_from_modem(modem, modemname, send_delay, answer, max, 2);  // One read attempt is 200ms
+
+      // check if it's the expected answer
+      if (expect && expect[0] && (regexec(&re, answer, (size_t) 0, NULL, 0) == 0))
+      {
+        got_timeout = 0;
+        put_command_timeouts = 0;
+        break;
+      }
+
+      timeoutcounter += 2;
+    }
+    // repeat until timout
+    while (timeoutcounter < timeout);
+
+    if (got_timeout)
+    {    
+      put_command_timeouts++;
+      if (expect && expect[0])
+        writelogfile(LOG_DEBUG, modemname, "put_command expected %s, timeout occurred.", expect);
     }
 
-    timeoutcounter+=2;
-  }
-  // repeat until timout
-  while (timeoutcounter<timeout);
+    strncpy(loganswer, answer, sizeof(loganswer) -1);
+    loganswer[sizeof(loganswer) -1] = 0;
+    cutspaces(loganswer);
+    cut_emptylines(loganswer);
 
-  if (got_timeout)
-  {    
-    put_command_timeouts++;
-    if (expect && expect[0])
-      writelogfile(LOG_DEBUG,modemname,"put_command expected %s, timeout occurred.", expect);
-  }
+    // 3.1:
+    if (log_single_lines)
+    {
+      char *p;
 
-  strncpy(loganswer,answer,sizeof(loganswer)-1);
-  loganswer[sizeof(loganswer)-1]=0;
-  cutspaces(loganswer);
-  cut_emptylines(loganswer);
-  writelogfile(LOG_DEBUG,modemname,"<- %s",loganswer);
+      while ((p = strchr(loganswer, '\r')))
+        *p = ' ';
+      while ((p = strchr(loganswer, '\n')))
+        *p = ' ';
+    }
+
+    writelogfile(LOG_DEBUG, modemname, "<- %s", loganswer);
+
+    // 3.1:
+    if (!got_timeout && strstr(loganswer, "ERROR"))
+    {
+      char *p;
+
+      p = get_gsm_error(loganswer);
+      if (*p)
+        writelogfile(LOG_DEBUG, modemname, "Explanation: %s", p);
+    }
+  }
 
   // Free memory used by regexp
-  regfree(&re);
+  if (regex_allocated)
+    regfree(&re);
 
-  return strlen(answer);
+  if (answer)
+    return strlen(answer);
+  return 1;
 }
 
 void setmodemparams(int modem, char* modemname, int rtscts,int baudrate) /* setup serial port */
@@ -412,6 +447,9 @@ void setmodemparams(int modem, char* modemname, int rtscts,int baudrate) /* setu
     case 300:    baudrate=B300; break;
     case 1200:   baudrate=B1200; break;
     case 2400:   baudrate=B2400; break;
+#ifdef B4800
+    case 4800:   baudrate=B4800; break;
+#endif
     case 9600:   baudrate=B9600; break;
     case 19200:  baudrate=B19200; break;
     case 38400:  baudrate=B38400; break;
@@ -424,7 +462,10 @@ void setmodemparams(int modem, char* modemname, int rtscts,int baudrate) /* setu
 #ifdef B230400
     case 230400: baudrate=B230400; break;
 #endif
-    default: writelogfile(LOG_ERR,modemname,"Baudrate not supported, using 9600"); baudrate=B9600;    
+
+    default:
+      writelogfile(LOG_ERR, modemname, "Baudrate %d not supported, using 9600", baudrate);
+      baudrate=B9600;
   }
   cfsetispeed(&newtio,baudrate);
   cfsetospeed(&newtio,baudrate);
@@ -432,35 +473,40 @@ void setmodemparams(int modem, char* modemname, int rtscts,int baudrate) /* setu
 }
 
 int initmodem(int device, int modem, char* modemname, int send_delay, int error_sleeptime, char* pin, char* initstring1, char* initstring2,
-              char* mode, char* smsc, int *check_network, int pinsleeptime, int pre_init)
+              char* smsc, int *check_network, int pinsleeptime, int pre_init)
 {
   char command[100];
   char answer[500];
   int retries=0;
-  int success=0;
   char *p;
   char *pre_initstring = "ATE0+CMEE=1\r";
 
+  // 3.1beta7: terminating is only checked in case of errors.
+
   writelogfile(LOG_INFO,modemname,"Checking if modem is ready");
   retries=0;
   do
   {
     retries++;
     put_command(modem,modemname,send_delay,"AT\r",answer,sizeof(answer),50,"(OK)|(ERROR)");
-    if ( strstr(answer,"OK")==NULL && strstr(answer,"ERROR")==NULL)
+    if (!strstr(answer, "OK") && !strstr(answer, "ERROR"))
     {
       if (terminate)
         return 7;
+
       // if Modem does not answer, try to send a PDU termination character
       put_command(modem,modemname,send_delay,"\x1A\r",answer,sizeof(answer),50,"(OK)|(ERROR)");
+
+      if (terminate)
+        return 7;
     }
-    if (terminate)
-      return 7;
   }
-  while ((retries<=10) && (! strstr(answer,"OK")));
-  if (! strstr(answer,"OK"))
+  while (retries <= 10 && !strstr(answer,"OK"));
+  if (!strstr(answer,"OK"))
   {
-    writelogfile0(LOG_ERR,modemname, tb_sprintf("Modem is not ready to answer commands"));
+    // 3.1: more detailed error message:
+    p = get_gsm_error(answer);
+    writelogfile0(LOG_ERR, modemname, tb_sprintf("Modem is not ready to answer commands%s%s", (*p)? ", " : "", p));
     alarm_handler0(LOG_ERR,modemname, tb);
     return 1;
   }
@@ -469,12 +515,8 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
   {
     writelogfile(LOG_INFO,modemname,"Pre-initializing modem");
     put_command(modem,modemname,send_delay,pre_initstring,answer,sizeof(answer),100,"(OK)|(ERROR)");
-    if (strstr(answer,"OK")==0)
-    {
+    if (!strstr(answer,"OK"))
       writelogfile(LOG_ERR,modemname,"Modem did not accept the pre-init string");
-      //alarm_handler(LOG_ERR,modemname,"Modem did not accept the init string");
-      //return 3;
-    }
   }
 
   if (pin[0])
@@ -496,15 +538,14 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
 
       // After a PIN is entered, some modems need some time before next commands are processed.
       if (pinsleeptime > 0)
-        if (t_sleep(pinsleeptime))
-          return 7;
+        t_sleep(pinsleeptime);
 
       put_command(modem,modemname,send_delay,"AT+CPIN?\r",answer,sizeof(answer),50, cpin_expect);
       if (strstr(answer,"+CPIN: SIM PIN") && !strstr(answer, "PIN2"))
       {
         writelogfile0(LOG_ERR,modemname, tb_sprintf("Modem did not accept this PIN"));
         alarm_handler0(LOG_ERR,modemname, tb);
-        abnormal_termination(0); //return 2;
+        abnormal_termination(0);
       }
       if (strstr(answer,"+CPIN: READY"))
         writelogfile(LOG_INFO,modemname,"PIN Ready");
@@ -514,7 +555,7 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
     {
       writelogfile0(LOG_CRIT,modemname, tb_sprintf("Your SIM is locked. Unlock it manually"));
       alarm_handler0(LOG_CRIT,modemname, tb);
-      abnormal_termination(0); //return 2;
+      abnormal_termination(0);
     }
 
     if (!strstr(answer, "+CPIN: READY"))
@@ -522,7 +563,7 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
       p = get_gsm_error(answer);
       writelogfile0(LOG_CRIT,modemname, tb_sprintf("PIN handling: expected READY, modem answered %s%s%s", cut_ctrl(answer), (*p)? ", " : "", p));
       alarm_handler0(LOG_CRIT,modemname, tb);
-      abnormal_termination(0); //return 2;
+      abnormal_termination(0);
     }
   }
 
@@ -544,7 +585,9 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
     while (retries < 2 && !strstr(answer,"OK"));
     if (strstr(answer,"OK")==0)
     {
-      writelogfile0(LOG_ERR,modemname, tb_sprintf("Modem did not accept the init string"));
+      // 3.1: more detailed error message:
+      p = get_gsm_error(answer);
+      writelogfile0(LOG_ERR, modemname, tb_sprintf("Modem did not accept the init string%s%s", (*p)? ", " : "", p));
       alarm_handler0(LOG_ERR,modemname, tb);
       return 3;
     }
@@ -563,9 +606,11 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
             return 7;
     }
     while (retries < 2 && !strstr(answer,"OK"));
-    if (strstr(answer,"OK")==0)
+    if (!strstr(answer,"OK"))
     {
-      writelogfile0(LOG_ERR,modemname, tb_sprintf("Modem did not accept the second init string"));
+      // 3.1: more detailed error message:
+      p = get_gsm_error(answer);
+      writelogfile0(LOG_ERR, modemname, tb_sprintf("Modem did not accept the second init string%s%s", (*p)? ", " : "", p));
       alarm_handler0(LOG_ERR,modemname, tb);
       return 3;
     }
@@ -573,6 +618,15 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
 
   if (*check_network)
   {
+    switch (wait_network_registration(modem, modemname, send_delay, check_network, 1, 100))
+    {
+      case -1:
+        return 4;
+ 
+      case -2:
+        return 7;
+    }
+/*
     writelogfile(LOG_INFO,modemname,"Checking if Modem is registered to the network");
     success=0;
     retries=0;
@@ -580,14 +634,18 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
     {
       retries++;
       put_command(modem,modemname,send_delay,"AT+CREG?\r",answer,sizeof(answer),100,"(\\+CREG:.*OK)|(ERROR)");
-      if (strstr(answer,"1"))
+
+      // 3.1:
+      // Second field is tested. Depending of a creg mode, there can be more and different information.
+      //if (strstr(answer,"1"))
+      if (strstr(answer, ",1"))
       {
         writelogfile(LOG_INFO,modemname,"Modem is registered to the network");
         success=1;
       }
-      else if (strstr(answer,"5"))
+      //else if (strstr(answer,"5"))
+      else if (strstr(answer, ",5"))
       {
-      	// added by Thomas Stoeckel
       	writelogfile(LOG_INFO,modemname,"Modem is registered to a roaming partner network");
 	success=1;
       }
@@ -612,8 +670,6 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
           return 7;
 	success=0;
       }
-      if (terminate)
-        return 7;
     }
     while ((success==0)&&(retries<10));
 
@@ -623,37 +679,29 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
       alarm_handler0(LOG_ERR,modemname, tb);
       return 4;
     }
+*/
   }
 
-  if (mode[0])
+  writelogfile(LOG_INFO,modemname,"Selecting PDU mode");
+  strcpy(command,"AT+CMGF=0\r");
+  retries=0;
+  do
   {
-    if ((strcmp(mode,"ascii")==0))
-    {
-      writelogfile(LOG_INFO,modemname,"Selecting ASCII mode");
-      strcpy(command,"AT+CMGF=1\r");
-    }
-    else
-    {
-      writelogfile(LOG_INFO,modemname,"Selecting PDU mode");
-      strcpy(command,"AT+CMGF=0\r");
-    }
-    retries=0;
-    do
-    {
-      retries++;
-      put_command(modem,modemname,send_delay,command,answer,sizeof(answer),50,"(OK)|(ERROR)");
-      if (strstr(answer, "ERROR"))
-        if (retries < 2)
-          if (t_sleep(error_sleeptime))
-            return 7;
-    }
-    while (retries < 2 && !strstr(answer,"OK"));
-    if (strstr(answer,"ERROR"))
-    {
-      writelogfile0(LOG_ERR,modemname, tb_sprintf("Error: Modem did not accept mode selection"));
-      alarm_handler0(LOG_ERR,modemname, tb);
-      return 5;
-    }
+    retries++;
+    put_command(modem,modemname,send_delay,command,answer,sizeof(answer),50,"(OK)|(ERROR)");
+    if (strstr(answer, "ERROR"))
+      if (retries < 2)
+        if (t_sleep(error_sleeptime))
+          return 7;
+  }
+  while (retries < 2 && !strstr(answer,"OK"));
+  if (strstr(answer,"ERROR"))
+  {
+    // 3.1: more detailed error message:
+    p = get_gsm_error(answer);
+    writelogfile0(LOG_ERR, modemname, tb_sprintf("Error: Modem did not accept mode selection%s%s", (*p)? ", " : "", p));
+    alarm_handler0(LOG_ERR,modemname, tb);
+    return 5;
   }
 
   if (smsc[0])
@@ -673,7 +721,9 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
     while (retries < 2 && !strstr(answer,"OK"));
     if (strstr(answer,"ERROR"))
     {
-      writelogfile0(LOG_ERR,modemname, tb_sprintf("Error: Modem did not accept SMSC"));
+      // 3.1: more detailed error message:
+      p = get_gsm_error(answer);
+      writelogfile0(LOG_ERR, modemname, tb_sprintf("Error: Modem did not accept SMSC%s%s", (*p)? ", " : "", p));
       alarm_handler0(LOG_ERR,modemname, tb);
       return 6;
     }
@@ -682,18 +732,38 @@ int initmodem(int device, int modem, char* modemname, int send_delay, int error_
   // 3.1beta7, 3.0.9: International Mobile Subscriber Identity is asked once.
   if (devices[device].identity[0] == 0)
   {
-    writelogfile(LOG_INFO,modemname,"Querying IMSI");
+    //writelogfile(LOG_INFO,modemname,"Querying IMSI");
     sprintf(command,"AT+CIMI\r");
     put_command(modem,modemname,send_delay,command, devices[device].identity, sizeof(devices[device].identity), 50,"(OK)|(ERROR)");
+
+    // 3.1:
+    if (strncmp(devices[device].identity, "AT+CIMI", 7) == 0)
+      strcpy(devices[device].identity, devices[device].identity +7);
+
+    // 3.1beta7: If CIMI is not supported, try CGSN
+    // TODO: is IMSI title still good?
+    if (strstr(devices[device].identity, "ERROR"))
+    {
+      sprintf(command,"AT+CGSN\r");
+      put_command(modem,modemname,send_delay,command, devices[device].identity, sizeof(devices[device].identity), 50,"(OK)|(ERROR)");
+
+      // 3.1:
+      if (strncmp(devices[device].identity, "AT+CGSN", 7) == 0)
+        strcpy(devices[device].identity, devices[device].identity +7);
+    }
+
     if (!strstr(devices[device].identity, "ERROR"))
     {
       if ((p = strstr(devices[device].identity, "OK")))
         *p = 0;
       cut_ctrl(devices[device].identity);
       cutspaces(devices[device].identity);
-      strcpy(devices[device].identity_header, "IMSI");
-      writelogfile(LOG_INFO,modemname,"%s: %s", devices[device].identity_header, devices[device].identity);
+      //strcpy(devices[device].identity_header, "IMSI");
+      //writelogfile(LOG_INFO,modemname,"%s: %s", devices[device].identity_header, devices[device].identity);
+      writelogfile(LOG_INFO,modemname,"IMSI: %s", devices[device].identity);
     }
+    else
+      writelogfile(LOG_INFO,modemname,"IMSI/CGSN not supported");
   }
 
   return 0;
@@ -843,3 +913,74 @@ int talk_with_modem(int device)
 
   return result;
 }
+
+// Return value:
+// -2 = terminated
+// -1 = modem is not registered
+// >= 0 = number of retries, modem is registered
+int wait_network_registration(int modem, char *modemname, int send_delay,
+                              int *check_network, int error_sleeptime, int retry_count)
+{
+  char answer[500];
+  int success = 0;
+  int retries = 0;
+
+  writelogfile(LOG_INFO, modemname, "Checking if Modem is registered to the network");
+
+  do
+  {
+    // 3.1: signal quality is logged:
+    if (retries > 0)
+      put_command(modem, modemname, send_delay, "AT+CSQ\r", answer, sizeof(answer), 100, "(OK)|(ERROR)");
+
+    put_command(modem, modemname, send_delay, "AT+CREG?\r", answer, sizeof(answer), 100, "(\\+CREG:.*OK)|(ERROR)");
+
+    // 3.1:
+    // Second field is tested. Depending of a creg mode, there can be more and different information.
+    //if (strstr(answer,"1"))
+    if (strstr(answer, ",1"))
+    {
+      writelogfile(LOG_INFO, modemname, "Modem is registered to the network");
+      success = 1;
+    }
+    //else if (strstr(answer,"5"))
+    else if (strstr(answer, ",5"))
+    {
+      writelogfile(LOG_INFO, modemname, "Modem is registered to a roaming partner network");
+      success = 1;
+    }
+    else if (strstr(answer,"ERROR"))
+    {
+      writelogfile(LOG_INFO, modemname, "Ignoring that modem does not support +CREG command.");
+      success = 1;
+      *check_network = 0;
+    }
+    else if (strstr(answer,"+CREG:"))
+    {
+      //writelogfile(LOG_NOTICE,modemname,"Modem is not registered, waiting %i sec. before retrying",error_sleeptime);
+      writelogfile(LOG_NOTICE,modemname, "MODEM IS NOT REGISTERED, WAITING %i SEC. BEFORE TRYING %i. TIME", error_sleeptime, retries +1);
+      if (t_sleep(error_sleeptime))
+        return -2;
+    }
+    else
+    {
+      writelogfile0(LOG_ERR, modemname, tb_sprintf("Error: Unexpected answer from Modem after +CREG?, waiting %i sec. before retrying", error_sleeptime));
+      alarm_handler0(LOG_ERR, modemname, tb);
+      if (t_sleep(error_sleeptime))
+        return -2;
+    }
+
+    if (!success)
+      retries++;
+  }
+  while (!success && retries < retry_count);
+
+  if (!success)
+  {
+    writelogfile0(LOG_ERR, modemname, tb_sprintf("Error: Modem is not registered to the network"));
+    alarm_handler0(LOG_ERR, modemname, tb);
+    return -1;
+  }
+
+  return retries;
+}
