@@ -5,6 +5,7 @@ http://smstools3.kekekasvi.com/
 
 Based on SMS Server Tools 2 from Stefan Frings
 http://www.meinemullemaus.de/
+SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
 from the author. You can redistribute it and/or modify it under the terms of
@@ -19,6 +20,10 @@ Either version 2 of the License, or (at your option) any later version.
 SOME DEFINITIONS ARE USED IN THE VERSION >= 3.1 ONLY.
 ************************************************** */
 
+#define DATETIME_DEFAULT "%y-%m-%d %H:%M:%S"
+
+#define CONCATENATED_DIR_FNAME "%s/%s-concatenated"
+
 #define PROVIDER 16 
 #define DEVICES 64
 #define NUMS 64
@@ -37,8 +42,11 @@ SOME DEFINITIONS ARE USED IN THE VERSION >= 3.1 ONLY.
 #define SIZE_SMSC 100
 #define SIZE_QUEUENAME 100
 #define SIZE_UDH_DATA 500
+#define SIZE_UDH_TYPE 4096
 #define SIZE_RR_CMD 513
 #define SIZE_MACROS 4096
+#define SIZE_HEADER 101
+#define SIZE_MESSAGEIDS 4096
 
 #include <limits.h>
 #include <sys/types.h>
@@ -46,6 +54,9 @@ SOME DEFINITIONS ARE USED IN THE VERSION >= 3.1 ONLY.
 char process_title[32];         // smsd for main task, name of a modem for other tasks.
 int process_id;                 // -1 for main task, all other have numbers starting with 0.
                                 // This is the same as device, can be used like devices[process_id]...
+int modem;                      // Handle for modem.
+
+int put_command_timeouts;
 
 typedef struct
 {
@@ -59,11 +70,12 @@ typedef struct
   char name[32];		// Name of the modem
   char device[PATH_MAX];	// Serial port name
   char identity[SIZE_TO];       // Identification asked from the modem (CIMI)
-  char identity_header[SIZE_TO];// Title of current identification (IMSI)
+  //char identity_header[SIZE_TO];// Title of current identification (IMSI)
   char queues[PROVIDER][32];	// Assigned queues
   int incoming; 		// Try to receive SMS. 0=No, 1=Low priority, 2=High priority
   int outgoing;                 // = 0 if a modem is not used to send messages.
   int report; 			// Ask for delivery report 0 or 1 (experimental state)
+  int phonecalls;               // Check for phonebook status for calls, 0 or 1
   char pin[16];			// SIM PIN
   int pinsleeptime;             // Number of seconds to sleep after a PIN is entered.
   char mode[10]; 		// Command set version old or new
@@ -92,13 +104,15 @@ typedef struct
   int message_limit;            // Limit counter for outgoing messages. 0 = no limit.
   int message_count_clear;      // Period to automatically clear message counter. Value is MINUTES. 
   int keep_open;                // 1 = do not close modem while idle.
-  char rr[PATH_MAX];            // Script/program which is run regularly.
-  int rr_interval;              // Number of seconds between running a regular_run script/progdam.
-  char rr_cmdfile[PATH_MAX];    // 
-  char rr_cmd[SIZE_RR_CMD];     //
-  char rr_logfile[PATH_MAX];
-  int rr_loglevel; // defaults to 5, LOG_NOTICE. Has only effect when a main log is used.
-  char rr_statfile[PATH_MAX];
+  char dev_rr[PATH_MAX];        // Script/program which is run regularly.
+  int dev_rr_interval;          // Number of seconds between running a regular_run script/progdam.
+  char dev_rr_cmdfile[PATH_MAX];// 
+  char dev_rr_cmd[SIZE_RR_CMD]; //
+  char dev_rr_logfile[PATH_MAX];
+  int dev_rr_loglevel; // defaults to 5, LOG_NOTICE. Has only effect when a main log is used.
+  char dev_rr_statfile[PATH_MAX];
+  char logfile[PATH_MAX];       // Name or Handle of Log File
+  int loglevel;                 // Log Level (9=highest). Verbosity of log file.
 } _device;
 
 // NOTE for regular run intervals: effective value is at least delaytime.
@@ -108,6 +122,8 @@ char d_spool[PATH_MAX];		// Spool directory
 char d_failed[PATH_MAX];	// Failed spool directory
 char d_incoming[PATH_MAX];	// Incoming spool directory
 char d_report[PATH_MAX];	// Incoming report spool directory
+char d_phonecalls[PATH_MAX];    // Incoming phonecalls data directory
+char d_saved[PATH_MAX];         // Directory for smsd's internal use, concatenation storage files etc.
 char d_sent[PATH_MAX];		// Sent spool directory
 char d_checked[PATH_MAX];	// Spool directory for checked messages (only used when no provider queues used)
 char eventhandler[PATH_MAX];	// Global event handler program or script
@@ -132,13 +148,15 @@ int internal_combine;           // 1 if multipart message is combined internally
 int keep_filename;              // 0 if unique filename is created to each directory when a message file is moved.
 int store_original_filename;    // 1 if an original filename is saved to message file when it's moved from
                                 // outgoing directory to spooler. Works together with keep_filename.
+int date_filename;              // 1 or 2 if YYYYMMDD is included to the filename of incoming message.
 char regular_run[PATH_MAX];     // Script/program which is run regularly.
 int regular_run_interval;       // Number of seconds between running a regular_run script/progdam.
 char admin_to[SIZE_TO];         // Destination number for administrative messages.
 int filename_preview;           // Number of chars of message text to concatenate to filename.
 int incoming_utf8;              // 1 if incoming files are saved using UTF-8 character set.
 int outgoing_utf8;              // 1 if outgoing files are automatically converted from UTF-8 to ISO and GSM.
-int log_charconv;               // 1 of character set conversion is logged.
+int log_charconv;               // 1 if character set conversion is logged.
+int log_single_lines;           // 1 if linefeeds are removed from the modem response to be logged.
 int message_count;              // Counter for sent messages. Multipart message is one message.
 
 int terminate;                  // The current process terminates if this is 1
@@ -168,7 +186,8 @@ char *startup_err_str;
 int startup_err_count;
 
 // Storage for PDU's:
-char *pdu_store;
+char *incoming_pdu_store;
+char *outgoing_pdu_store;
 
 // Storage for getfile errors:
 char *getfile_err_store;
@@ -178,6 +197,14 @@ char tb[1024];
 
 int os_cygwin;                  // 1 if we are on Cygwin.
 
+char language_file[PATH_MAX];   // File name of translated headers.
+char yes_chars[SIZE_HEADER];    // Characters which mean "yes" in the yesno() question.
+char no_chars[SIZE_HEADER];     // See details inside read_translation() function.
+char yes_word[SIZE_HEADER];     // "yes" printed as an output.
+char no_word[SIZE_HEADER];      // "no"
+char datetime_format[SIZE_HEADER]; // strftime format string for time stamps (not inside status reports).
+int translate_incoming;         // 0 if incoming message headers are NOT transtaled.
+
 /* initialize all variable with default values */
 
 void initcfg();
@@ -207,7 +234,7 @@ void help();
 
 void parsearguments(int argc,char** argv);
 
-int startup_check(void);
+int startup_check(int result);
 
 void abnormal_termination(int all);
 
